{*****************************************************}
{* 4-bit-full-adder unit for LogicGates3 application *}
{*****************************************************}

unit adder4f;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls, StdCtrls, Grids, common;

type
  {***********}
  { TfAdder4f }
  {***********}
  TfAdder4f = class(TForm)
    Shape1: TShape;
    StaticText1: TStaticText;
    Label1, Label2, Label8, Label9, Label10, Label18: TLabel;
    Shape20, Shape21, Shape22, Shape23, Shape25, Shape28, Shape29, Shape32: TShape;
    Shape35, Shape36, Shape37, Shape38, Shape39, Shape45, Shape50, Shape51: TShape;
    Shape52, Shape53, Shape54, Shape55, Shape57, Shape58, Shape60, Shape61: TShape;
    Shape62, Shape63, Shape64, Shape65, Shape66, Shape67, Shape68, Shape69: TShape;
    Shape70, Shape71, Shape72, Shape73, Shape74, Shape75, Shape76, Shape77: TShape;
    Shape78, Shape79, Shape80, Shape81, Shape82, Shape83, Shape84, Shape85: TShape;
    Shape86, Shape87, Shape88, Shape89, Shape90, Shape91, Shape92, Shape93: TShape;
    Shape94, Shape95, Shape96, Shape97, Shape98, Shape99, Shape100, Shape101: TShape;
    Shape102, Shape103, Shape104, Shape105, Shape106, Shape107, Shape108, Shape109: TShape;
    Shape110, Shape111, Shape112, Shape113, Shape114, Shape115, Shape116, Shape117: TShape;
    Shape118, Shape119, Shape120, Shape121, Shape122, Shape123, Shape124: TShape;
    Image2, Image3, Image4, Image5: TImage;
    laA1, laA2, laA4, laA8, laB1, laB2: TLabel;
    laB4, laB8, laS1, laS2, laS4, laS8: TLabel;
    imSwitchA1, imSwitchA2, imSwitchA4, imSwitchA8: TImage;
    imSwitchB1, imSwitchB2, imSwitchB4, imSwitchB8: TImage;
    shSwitchA1a, shSwitchA1b, shSwitchA2a, shSwitchA2b: TShape;
    shSwitchA4a, shSwitchA4b, shSwitchA8a, shSwitchA8b: TShape;
    shSwitchB1a, shSwitchB1b, shSwitchB2a, shSwitchB2b: TShape;
    shSwitchB4a, shSwitchB4b, shSwitchB8a, shSwitchB8b: TShape;
    shLedA1, shLedA2, shLedA4, shLedA8: TShape;
    shLedB1, shLedB2, shLedB4, shLedB8: TShape;
    shLedS1, shLedS2, shLedS4, shLedS8, shLedC: TShape;
    sgAdd: TStringGrid;
    Image1: TImage;
    btClose: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure imSwitchA1Click(Sender: TObject);
    procedure imSwitchA2Click(Sender: TObject);
    procedure imSwitchA4Click(Sender: TObject);
    procedure imSwitchA8Click(Sender: TObject);
    procedure imSwitchB1Click(Sender: TObject);
    procedure imSwitchB2Click(Sender: TObject);
    procedure imSwitchB4Click(Sender: TObject);
    procedure imSwitchB8Click(Sender: TObject);
    procedure btCloseClick(Sender: TObject);
  private
    aLeds: TLeds;
    aSwitches: TSwitches;
    bSwitches, bLeds: TStatus;
  end;

const
  // Switches description: Data = objects positions (transparent image, long bar shape, short bar shape), horiz. closing move, vert. closing move
  DSwitches: array[0..7] of TSwitchData = (
    (Name: 'SwitchA1';
     Data: (512, 534, 514, 0, 1)),
    (Name: 'SwitchA2';
     Data: (378, 400, 380, 0, 1)),
    (Name: 'SwitchA4';
     Data: (246, 268, 248, 0, 1)),
    (Name: 'SwitchA8';
     Data: (112, 134, 114, 0, 1)),
    (Name: 'SwitchB1';
     Data: (577, 599, 579, 0, 1)),
    (Name: 'SwitchB2';
     Data: (444, 466, 446, 0, 1)),
    (Name: 'SwitchB4';
     Data: (310, 333, 313, 0, 1)),
    (Name: 'SwitchB8';
     Data: (177, 200, 180, 0, 1))
  );

var
  fAdder4f: TfAdder4f;

implementation

{$R *.lfm}

{ 4-bit-full-adder: Set switch position; do binary addition; turn LEDs on/off; fill-in addition-table }

procedure SwitchAndAdd(DSwitches: array of TSwitchData; Switches: TSwitches; var SwitchStatus: TStatus; Leds: TLeds; var LedStatus: TStatus; IX: Integer);

const
  Bits: array[False..True] of Integer = (0, 1);
  HexDigits: array[0..15] of Char = ('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F' );

var
  Exp, Hex1, Hex2, I: Integer;
  A, B, C, Sout, Cout, S1, S2, C1, C2: Boolean;
  Dec: array[0..2] of Integer;

begin
  // Change current input from 0 to Vs (and vice-versa)
  SwitchStatus[IX] := not SwitchStatus[IX];
  // Corresponding LED status
  LedStatus[IX] := SwitchStatus[IX];
  // Set switch positions
  SwitchesOnOff(DSwitches, Switches, SwitchStatus);
  // Do the addition, using 4 1-bit full-adders, as is done in a real circuit:
  // The sum output of a given adder will the sum value for that bit,
  // the carry output being used as input for the subsequent adder
  Cout := False;                                                               // Cin for first adder set to 0V
  for I := 0 to 3 do begin
    A := LedStatus[I]; B := LedStatus[I + 4];                                  // actual bit for A and B input
    C := Cout;                                                                 // Cin = previous Cout
    // 1-bit-full-adder binary addition
    Adder(A, B, C, S1, C1, S2, C2, Sout, Cout);
    // Actual bit sum LED status
    LedStatus[I + 8] := SOut;
  end;
  // Last Cout is carry value for 4-bit operation: Cout LED status
  LedStatus[12] := Cout;
  // Turn all LEDs on/off
  LedsOnOff(Leds, LedStatus);
  // Fill-in binary values and calculate decimal values
  Dec[0] := 0; Dec[1] := 0; Dec[2] := 0; Exp := 1;
  for I := 0 to 3 do begin
    fAdder4f.sgAdd.Cells[5 - I, 1] := ' ' + IntToStr(Bits[LedStatus[I]]);
    fAdder4f.sgAdd.Cells[5 - I, 2] := ' ' + IntToStr(Bits[LedStatus[I + 4]]);
    fAdder4f.sgAdd.Cells[5 - I, 3] := ' ' + IntToStr(Bits[LedStatus[I + 8]]);
    Dec[0] += Bits[LedStatus[I]] * Exp; Dec[1] += Bits[LedStatus[I + 4]] * Exp; Dec[2] += Bits[LedStatus[I + 8]] * Exp;
    Exp *= 2;
  end;
  // Display the carry only if it is 1
  if LedStatus[12] then
    fAdder4f.sgAdd.Cells[1, 3] := ' ' + IntToStr(Bits[LedStatus[12]])
  else
    fAdder4f.sgAdd.Cells[1, 3] := '';
  Dec[2] += Bits[LedStatus[12]] * 16;
  // Fill-in decimal and (here calculated) hexadecimal values
  for I := 0 to 2 do begin
    fAdder4f.sgAdd.Cells[7, I + 1] := '  ' + IntToStr(Dec[I]);
    if Dec[I] < 10 then
      fAdder4f.sgAdd.Cells[7, I + 1] := ' ' + fAdder4f.sgAdd.Cells[7, I + 1];
    Hex1 := Dec[I] div 16; Hex2 := Dec[I] mod 16;
    fAdder4f.sgAdd.Cells[6, I + 1] := '  ' + HexDigits[Hex1] + HexDigits[Hex2];
  end;
end;

{***********}
{ TfAdder4f }
{***********}

{ Application start: Initialisation }

procedure TfAdder4f.FormCreate(Sender: TObject);

begin
  // Arrays with switches (transparent image and bar shapes) and their "on/off" status
  SetLength(aSwitches, 8); SetLength(bSwitches, 8);
  aSwitches[0].Switch0 := imSwitchA1; aSwitches[0].Switch1 := shSwitchA1a; aSwitches[0].Switch2 := shSwitchA1b;
  aSwitches[1].Switch0 := imSwitchA2; aSwitches[1].Switch1 := shSwitchA2a; aSwitches[1].Switch2 := shSwitchA2b;
  aSwitches[2].Switch0 := imSwitchA4; aSwitches[2].Switch1 := shSwitchA4a; aSwitches[2].Switch2 := shSwitchA4b;
  aSwitches[3].Switch0 := imSwitchA8; aSwitches[3].Switch1 := shSwitchA8a; aSwitches[3].Switch2 := shSwitchA8b;
  aSwitches[4].Switch0 := imSwitchB1; aSwitches[4].Switch1 := shSwitchB1a; aSwitches[4].Switch2 := shSwitchB1b;
  aSwitches[5].Switch0 := imSwitchB2; aSwitches[5].Switch1 := shSwitchB2a; aSwitches[5].Switch2 := shSwitchB2b;
  aSwitches[6].Switch0 := imSwitchB4; aSwitches[6].Switch1 := shSwitchB4a; aSwitches[6].Switch2 := shSwitchB4b;
  aSwitches[7].Switch0 := imSwitchB8; aSwitches[7].Switch1 := shSwitchB8a; aSwitches[7].Switch2 := shSwitchB8b;
  // Arrays with LEDs (shape) and their "on/off" status
  SetLength(aLeds, 13); SetLength(bLeds, 13);
  aLeds[0] := shLedA1; aLeds[1] := shLedA2; aLeds[2] := shLedA4; aLeds[3] := shLedA8;
  aLeds[4] := shLedB1; aLeds[5] := shLedB2; aLeds[6] := shLedB4; aLeds[7] := shLedB8;
  aLeds[8] := shLedS1; aLeds[9] := shLedS2; aLeds[10] := shLedS4; aLeds[11] := shLedS8; aLeds[12] := shLedC;
  // Use subscripts in input/output labels
  laA1.Caption := 'A' + SUB_Digits[1]; laA2.Caption := 'A' + SUB_Digits[2];
  laA4.Caption := 'A' + SUB_Digits[4]; laA8.Caption := 'A' + SUB_Digits[8];
  laB1.Caption := 'B' + SUB_Digits[1]; laB2.Caption := 'B' + SUB_Digits[2];
  laB4.Caption := 'B' + SUB_Digits[4]; laB8.Caption := 'B' + SUB_Digits[8];
  laS1.Caption := 'S' + SUB_Digits[1]; laS2.Caption := 'S' + SUB_Digits[2];
  laS4.Caption := 'S' + SUB_Digits[4]; laS8.Caption := 'S' + SUB_Digits[8];
end;

{ Application show-up: Initialize 4-bit-full-adder simulation }

procedure TfAdder4f.FormActivate(Sender: TObject);

var
  I: Integer;

begin
  // All switches (inputs) to 0
  for I := 0 to Length(bSwitches) - 1 do
    bSwitches[I] := False;
  SwitchesOnOff(DSwitches, aSwitches, bSwitches);
  // All LEDs off
  for I := 0 to Length(bLeds) - 1 do
    bLeds[I] := False;
  LedsOnOff(aLeds, bLeds);
end;

{ Switch "A1, A2..." / "B1, B2..." activated: Do 4-bit full-adder simulation for actual switch positions }

procedure TfAdder4f.imSwitchA1Click(Sender: TObject);

begin
  SwitchAndAdd(DSwitches, aSwitches, bSwitches, aLeds, bLeds, 0);
end;

procedure TfAdder4f.imSwitchA2Click(Sender: TObject);

begin
  SwitchAndAdd(DSwitches, aSwitches, bSwitches, aLeds, bLeds, 1);
end;

procedure TfAdder4f.imSwitchA4Click(Sender: TObject);

begin
  SwitchAndAdd(DSwitches, aSwitches, bSwitches, aLeds, bLeds, 2);
end;

procedure TfAdder4f.imSwitchA8Click(Sender: TObject);

begin
  SwitchAndAdd(DSwitches, aSwitches, bSwitches, aLeds, bLeds, 3);
end;

procedure TfAdder4f.imSwitchB1Click(Sender: TObject);

begin
  SwitchAndAdd(DSwitches, aSwitches, bSwitches, aLeds, bLeds, 4);
end;

procedure TfAdder4f.imSwitchB2Click(Sender: TObject);

begin
  SwitchAndAdd(DSwitches, aSwitches, bSwitches, aLeds, bLeds, 5);
end;

procedure TfAdder4f.imSwitchB4Click(Sender: TObject);

begin
  SwitchAndAdd(DSwitches, aSwitches, bSwitches, aLeds, bLeds, 6);
end;

procedure TfAdder4f.imSwitchB8Click(Sender: TObject);

begin
  SwitchAndAdd(DSwitches, aSwitches, bSwitches, aLeds, bLeds, 7);
end;

{ Button "Close": Close the window }

procedure TfAdder4f.btCloseClick(Sender: TObject);

begin
  Close;
end;

end.

