{*****************************************}
{* Main unit for LogicGates3 application *}
{*****************************************}

// Implementation of the interactive graphics:
//   - the circuit layout is constituted by a mixture of shapes and images
//   - the LEDs are colored ellipses; different color styles are used for on/off status
//   - the movement of the switches is done by changing the horizontal or vertical position (LEFT or TOP property) of the corresponding shapes
//   - the switch shapes being very small and difficult to click by the user, I used a transperant image, placed over them;
//     the OnClick event of the image is used to trigger the actions to be done when the corresponding switch has been clicked.

unit adders;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, Menus, StdCtrls, ExtCtrls, adder1h, adder1f, adder4f, common;

type
  {**********}
  { TfAdders }
  {**********}
  TfAdders = class(TForm)
    mMenu: TMainMenu;
    mFile, mFileExit: TMenuItem;
    mHelp, mHelpAbout: TMenuItem;
    StaticText1: TStaticText;
    Memo1, Memo2, Memo3: TMemo;
    Image1, Image2, Image3: TImage;
    btAdder1h: TButton;
    btAdder1f: TButton;
    btAdder4f: TButton;
    procedure FormCreate(Sender: TObject);
    procedure mFileExitClick(Sender: TObject);
    procedure mHelpAboutClick(Sender: TObject);
    procedure btAdder1hClick(Sender: TObject);
    procedure btAdder1fClick(Sender: TObject);
    procedure btAdder4fClick(Sender: TObject);
  end;

var
  fAdders: TfAdders;

implementation

{$R *.lfm}

{**********}
{ TfAdders }
{**********}

{ Application start: Initialisation }

procedure TfAdders.FormCreate(Sender: TObject);

var
  SA, SB: string;

begin
  SA := 'A' + SUB_Digits[1] + 'A' + SUB_Digits[2] + 'A' + SUB_Digits[4];
  SB := 'B' + SUB_Digits[1] + 'B' + SUB_Digits[2] + 'B' + SUB_Digits[4];
  Memo3.Text := StringReplace(Memo3.Text, 'A1A2A4', SA, []);
  Memo3.Text := StringReplace(Memo3.Text, 'B1B2B4', SB, []);
end;

{ Menu item "File > Exit": Exit application }

procedure TfAdders.mFileExitClick(Sender: TObject);

begin
  Close;
end;

{ Menu item "Help > About": Display application about }

procedure TfAdders.mHelpAboutClick(Sender: TObject);

var
  S: string;

begin
  S := 'Electronics simulation.' + LineEnding;
  S += 'Using logic gates to make binary adder circuits.' + LineEnding + LineEnding;
  S += 'Version 1.1.1, © allu, December 2019 - September 2021.';
  MessageDlg('About "LogicGates3"', S, mtInformation, [mbOK], 0);
end;

{ Button "1-bit-half-adder": Run 1-bit-half-adder simulation }

procedure TfAdders.btAdder1hClick(Sender: TObject);

begin
  fAdder1h.ShowModal;
end;

{ Button "1-bit-full-adder": Run 1-bit-full-adder simulation }

procedure TfAdders.btAdder1fClick(Sender: TObject);

begin
  fAdder1f.ShowModal;
end;

{ Button "4-bit-full-adder": Run 4-bit-full-adder simulation }

procedure TfAdders.btAdder4fClick(Sender: TObject);

begin
  fAdder4f.ShowModal;
end;

end.

